@@ -1,7 +1,14 @@
 from sven.exc import *
-from sven.backend import SvnAccess
-from sven.bzr import BzrAccess
+try:
+    from sven.backend import SvnAccess
+except ImportError:
+    SvnAccess = None
+try:
+    from sven.bzr import BzrAccess
+except ImportError:
+    BzrAccess = None
 from webob import Request, Response, exc
+import time, datetime
 
 def qs(dict):
     qs = ''
@@ -47,30 +54,13 @@ class SvnWikiView(object):
         return self.backend(self.checkout_dir)
 
     def get_response(self, template, data, content_type=None):
-        #if template != 'edit.html':
-        #    return Response(body=data['body'],
-        #                    content_type=data.get('mimetype',
-        #                                          'text/html'))
-
         data['mimetype'] = content_type = data.get('mimetype') or data.get('mimetype') or 'text/html'
 
-        if template != 'view.html':
-            content = self.template_loader(template, data)
-            content_type='text/html'
-            res = Response(content_type=content_type, body=content)
-        elif content_type == 'text/html':
-            body = data['body']
-            body = body.replace('\n', "<br/>")
-            body += "<div><a href='?view=edit'>edit file</a></div>"
-            res = Response(content_type=content_type, body=body)
-        else:
-            res = Response(content_type=content_type, body=data['body'])
-
-        res.content_length = len(res.body)
+        content = self.template_loader(template, data)
+        content_type='text/html'
+        res = Response(content_type=content_type, body=content)
         return res
 
-
-
     def __call__(self, environ, start_response):        
         request = Request(environ)
         request.path_info = request.path_info.rstrip('/')
@@ -83,26 +73,27 @@ class SvnWikiView(object):
         if not save:
             return None
 
-        contents, message, mimetype, res = save(request)
+        contents, message, metadata, res = save(request)
+
         self.svn(request).write(request.path_info, 
-                                contents, message, mimetype)
+                                contents, message, metadata=metadata)
+        x = self.svn(request)
+        uri = request.path_info
         return res
 
     def maybe_editform(self, request):
         try:
-            q = self.svn(request).read(request.path_info)
-            content = q['body']
-            mimetype = q['mimetype']
+            content = self.svn(request).read(request.path_info)
+            metadata = {}
         except NotAFile:
             return exc.HTTPBadRequest("Cannot edit directories")
         except NoSuchResource:
             return self.editor.new(request)
 
-        edit = self.editor.match_edit(request, content, mimetype)
-        if not edit:
-            return None
+        x = self.svn(request)
+        uri = request.path_info
 
-        return edit(request, content, mimetype)
+        return self.editor.editform(request, content, metadata)
 
     def handle_request(self, request):
         if request.method == "POST":
@@ -113,6 +104,9 @@ class SvnWikiView(object):
             edit = self.maybe_editform(request)
             if edit is not None: return edit
 
+        if request.method == "HEAD" and not request.GET.get('view'):
+            return self.default_view_head(request)
+
         if request.method != "GET":
             return exc.HTTPMethodNotAllowed("GET", "POST")
 
@@ -138,11 +132,18 @@ class SvnWikiView(object):
 
         for obj in contents:
             obj['full_href'] = request.script_name + obj['href']
+            timestamp = obj['fields']['timestamp']
+            from wsgiref.handlers import format_date_time
+            obj['fields']['timestamp'] = \
+                format_date_time(timestamp)
         contents = {'globs': contents}
         contents['title'] = request.path_info.rstrip('/')
         
         # put the raw form of the resource in the wsgi environment
         request.environ['svenweb.resource'] = dict(contents)
+
+        if request.GET.has_key('atom'):
+            return self.get_response('history.atom', contents, "application/atom+xml")
         return self.get_response('history.html', contents, "text/html")
 
     def index_view(self, request):
@@ -157,6 +158,8 @@ class SvnWikiView(object):
             request.GET['version'] = str(e.last_change)
             return redirect(request, perm=True)
 
+        uri = request.path_info
+
         prefix = request.script_name.rstrip('/')
 
         for item in contents:
@@ -171,6 +174,43 @@ class SvnWikiView(object):
 
         return self.get_response('index.html', contents, "text/html")
 
+    def default_view_head(self, request):
+        version =request.GET.get('version', None)
+
+        if request.path_info == '':
+            request.path_info = '/'
+            request.GET['view'] = 'index'
+            return redirect(request)
+
+        try:
+            last_changed_rev = self.svn(request).last_changed_rev(request.path_info, version)
+            if last_changed_rev and not version:
+                # note: this case IS different from the ResourceUnchanged exception
+                #       below, because here the redirect is temporary.
+                # in fact, this can be considered the default behavior of the "default view"
+                request.GET['version'] = str(last_changed_rev)
+                if request.GET.has_key("latest-version"):
+                    version = last_changed_rev
+                else:
+                    return redirect(request)
+        except NotAFile:
+            raise
+            request.GET['view'] = 'index'
+            return redirect(request)
+        except NoSuchResource:
+            return exc.HTTPNotFound()
+            request.GET['view'] = 'edit'
+            if request.GET.has_key('version'):
+                del request.GET['version']
+            return redirect(request)
+        except ResourceUnchanged, e:
+            request.GET['version'] = str(e.last_change)
+            return redirect(request)
+
+        res = Response()
+        res.headers['X-Svenweb-Version'] = version
+        return res
+
     def default_view(self, request):
         #default case (just a plain old view)
         version =request.GET.get('version', None)
@@ -187,32 +227,71 @@ class SvnWikiView(object):
                 #       below, because here the redirect is temporary.
                 # in fact, this can be considered the default behavior of the "default view"
                 request.GET['version'] = str(last_changed_rev)
-                return redirect(request)
-            contents = self.svn(request).read(request.path_info, version)
+                if request.GET.has_key("latest-version"):
+                    version = last_changed_rev
+                else:
+                    return redirect(request)
+            data = self.svn(request).read(request.path_info, version)
         except NotAFile:
             request.GET['view'] = 'index'
             return redirect(request)
         except NoSuchResource:
-            request.GET['view'] = 'edit'
-            if request.GET.has_key('version'):
-                del request.GET['version']
-            return redirect(request)
+            # We might have gotten here because the page 
+            # did not yet exist at the given revision.
+            # So, let's check whether it has any revisions
+            # in the log at all.
+            try:
+                revs = self.svn(request).revisions(request.path_info)
+
+            except NoSuchResource:
+                # OK, it really doesn't exist.
+                request.GET['view'] = 'edit'
+                if request.GET.has_key('version'):
+                    del request.GET['version']
+                return redirect(request)
+
+            # It does exist, at some point in the future.
+            assert revs and len(revs)
+            first_rev = revs[-1]
+            error = "<h1>Page not yet born at revision %s</h1>" % version
+            request.GET['version'] = first_rev
+            redirected = redirect(request)
+            error += "<div>Perhaps try <a href='%s'>its first revision</a> instead" % redirected.location
+            return exc.HTTPNotFound(body=error)
+
         except ResourceUnchanged, e:
             request.GET['version'] = str(e.last_change)
             return redirect(request)
-        
-        content = contents['body']
-        mimetype = contents['mimetype']
 
-        view = self.viewer.match_view(request, content, mimetype)
+        except FutureRevision, e:
+            error = "<h1>No such revision %s</h1>" % e.rev
+            last_changed_rev = self.svn(request).last_changed_rev(request.path_info, version)
+            request.GET['version'] = last_changed_rev
+            redirected = redirect(request)
+            error += "<div>Perhaps try <a href='%s'>its last revision</a> instead" % redirected.location
+            return exc.HTTPNotFound(body=error)
+
+        contents = {'contents': data}
+        mimetype = "text/html"
+
+        view = self.viewer.match_view(request, data, mimetype)
         if view:
-            return view(request, content)
+            return view(request, data)
 
         contents['prev_href'] = "%s?version=%d" % (request.path_info,
                                                    int(
                 request.GET['version']) - 1)
 
-        contents['title'] = request.path_info.rstrip('/')
+        x = self.svn(request)
+        uri = request.path_info
+
         contents['format'] = request.GET.get('format')
 
-        return self.get_response('view.html', contents, contents['mimetype'] or 'text/html')
+        res = Response(
+            content_type=mimetype,
+            body=data)
+        res.content_length = len(res.body)
+
+        res.headers['X-Svenweb-Version'] = version
+
+        return res
